# RW API microservice utility for Python+Flask applications

[![Build Status](https://travis-ci.com/resource-watch/rw-api-microservice-python.svg?branch=main)](https://travis-ci.com/resource-watch/rw-api-microservice-python)
[![Test Coverage](https://api.codeclimate.com/v1/badges/fe857a0082ab7d0bbd64/test_coverage)](https://codeclimate.com/github/resource-watch/rw-api-microservice-python/test_coverage)

Library to register and integrate microservices in the [RW API](https://api.resourcewatch.org/).


## Requirements

This library is tested to work with following Python versions:
- 3.11.4
- 3.10.12

Additionally, it requires the following Python libraries: 
- Flask (tested with v2.2 and v2.3)
- requests
- boto3

## Install

```shell
pip install RWAPIMicroservicePython
```

## Testing

```shell
tox
```

These tests run on multiple python versions in parallel. You may want/need to use something like `pyenv` to support the underlying version handling. If you are using `pyenv virtualenv` be sure to deactivate any envs before calling `tox`.

## Use in microservice

To bootstrap your microservice, use:

```python
import RWAPIMicroservicePython
from flask import Flask

app = Flask(__name__)

RWAPIMicroservicePython.register(
    app=app,
    gateway_url='https://control-tower.your.domain',
    token='microserviceTokenForControlTower',
    aws_cloud_watch_log_stream_name='nameOfTheAWSCloudwatchLogStream',
    aws_region='youAWSRegion'
)
```

To make a request to another microservice, use:

```python
from RWAPIMicroservicePython import request_to_microservice

def execute():
    config = {
        'uri': '/v1/dataset/1234',
        'method': 'POST',
        'api_key': '<api key from the request>',<
        'body': {'key': 'value'}
    }
    response = request_to_microservice(**config)
    if not response or response.get('errors'):
        raise Exception(message='Dataset not found')

    dataset = response.get('data', None).get('attributes', None)
    return dataset
```

## Configuration

These are the values you'll need to provide when using this library:

- app: the [Flask](https://flask.palletsprojects.com) `app`.
- gateway_url: the URL of the API as a whole, where all other services will be reachable.
- token: JWT token to use on calls to other services.
- aws_cloud_watch_log_stream_name: Name of the AWS CloudWatch stream under which to log API activity
- aws_region: AWS region

